module PackageCloud
  module CLI
    class Entry < Base
      desc "repository SUBCMD ...ARGS", "manage repositories"
      subcommand "repository", Repository

      desc "distro SUBCMD ...ARGS", "manage repositories"
      subcommand "distro", Distro

      desc "master_token SUBCMD ...ARGS", "manage master tokens"
      subcommand "master_token", MasterToken


      desc "push user/repo[/distro/version] /path/to/packages",
           "push package(s) to repository (in distro/version if required)"
      def push(repo, package_file, *package_files)
        ARGV.clear # otherwise gets explodes
        package_files << package_file

        exts = package_files.map { |f| f.split(".").last }.uniq

        if package_files.length > 1 && exts.length > 1
          abort("You can't push multiple packages of different types at the same time.\nFor example, use *.deb to push all your debs at once.".red)
        end

        invalid_packages = package_files.select do |f|
          !["gem", "deb", "rpm"].include?(f.split(".").last)
        end

        if invalid_packages.any?
          message = "I don't know how to push these packages:\n\n".red
          invalid_packages.each do |p|
            message << "  #{p}\n"
          end
          message << "\npackage_cloud only supports debs, gems, and rpms.".red
          abort(message)
        end

        if exts.first == "gem" && package_files.length > 1
          answer = get_valid("Are you sure you want to push #{package_files.length} packages? (y/n)") do |s|
            s == "y" || s == "n"
          end

          if answer != "y"
            puts "Aborting...".red
          end
        end

        validator = Validator.new(client)
        dist_id   = validator.distribution_id(repo, package_files, exts.first)

        # strip os/dist
        repo = repo.split("/")[0..1].join("/")

        print "Looking for repository at #{repo}... "
        repo = client.repository(repo)
        print "success!\n"
        package_files.each do |f|
          print "Pushing #{f}... "
          repo.create_package(f, dist_id)
        end
      end

      desc "version",
           "print version information"
      def version
        puts "package_cloud CLI #{VERSION}\nSee https://packagecloud.io/docs#cli for more details."
      end

      private
        def get_valid(prompt)
          selection = ""
          times = 0
          until yield(selection)
            if times > 0
              puts "#{selection} is not a valid selection."
            end
            print "#{prompt}: "
            selection = ::Kernel.gets.chomp
            times += 1
          end

          selection
        end
    end
  end
end
